"""ccg_django_utils.conf -- Django settings util functions.

This module contains helper functions for working with environment
variable configuration of a Django app.

The goal is to use simple key-value settings so that the Django app
can be deployed in multiple scenarios with minimum fussing around
settings files. This simplification also helps makes the dev settings
more like production settings.

When deploying your application, you need to put the settings into the
web application's environment. Typically you would call
setup_prod_env() or setup_config_env() from both the WSGI handler and
the management script. These functions load settings from a simple
config file.

The Django settings module then gets configuration from the
environment. The ``EnvConfig`` class provides a methods to do this.


References:
 * http://12factor.net/config
 * http://12factor.net/dev-prod-parity
 * https://devcenter.heroku.com/articles/getting-started-with-django#django-settings
 * https://github.com/twoscoops/django-twoscoops-project/blob/2910cb96/project_name/project_name/settings/production.py

"""

from __future__ import unicode_literals
import os
import sys
from six.moves.configparser import ConfigParser
from six import StringIO

__all__ = ["EnvConfig", "setup_prod_env", "setup_config_env"]

def setup_prod_env(project_name, config_file=None, package_name=None):
    """
    Sets environment variables for the web app according to conventions.
    """
    package_name = package_name or project_name

    os.environ.setdefault('DJANGO_SETTINGS_MODULE', '%s.prodsettings' % package_name)
    os.environ.setdefault('PYTHON_EGG_CACHE', '/tmp/.python-eggs')

    # ccg.utils.webhelpers expects SCRIPT_NAME to be a path such as /project-name
    # This is a fallback value if the variable isn't set through wsgi.
    os.environ.setdefault("SCRIPT_NAME", "/" + project_name)

    os.environ["PROJECT_NAME"] = project_name
    os.environ["PRODUCTION"] = "1"

    # set up the environment with settings loaded from a config file.
    config_file = config_file or "/etc/%s/%s.conf" % (project_name, project_name)
    setup_config_env(config_file)

def setup_config_env(config_file):
    """
    Loads settings from a key-value config file into environment
    variables.
    """
    for key, val in load_config(config_file):
        os.environ[key.upper()] = val

def load_config(filename):
    section = "root"

    try:
        config_text = "[%s]\n%s" % (section, open(filename).read())
    except IOError as e:
        sys.stderr.write("load_config: %s\n" % e)
        config_text = "[%s]\n" % section

    config = ConfigParser()
    config.readfp(StringIO(config_text))

    return config.items(section)

class EnvConfig(object):

    def get(self, setting, default=None):
        """
        Get the environment setting, return a default value, or raise
        an exception.
        Values used by this function will likely come from a conf file in /etc.
        """

        # simple conversion of values to numbers or bool
        if isinstance(default, bool):
            conv = lambda v: v.lower()[0:1] in ("1", "y", "t")
        elif isinstance(default, (int, float)):
            conv = type(default)
        else:
            conv = lambda x: x

        return self._get_setting(setting, default, conv)

    def getlist(self, setting, default=None):
        return self._get_setting(setting, default, lambda x: x.split())

    def _get_setting(self, setting, default, conv):
        try:
            return conv(self[setting])
        except KeyError:
            if default is None:
                from django.core.exceptions import ImproperlyConfigured
                error_msg = "Set the %s env variable" % setting
                raise ImproperlyConfigured(error_msg)
            else:
                return default

    def __getitem__(self, setting):
        # make settings case-insensitive
        setting = setting.upper()
        return os.environ[setting]

    def __hasitem__(self, setting):
        return setting.upper() in os.environ

    def __delitem__(self, setting):
        setting = setting.upper()
        if setting in os.environ:
            del os.environ[setting]

    def get_db_engine(self, setting, default):
        """
        Gets the environment setting for database engine and maps it to a
        Django database backend class.
        Possible values are: pgsql, mysql, sqlite3.
        """
        engines = {
            'pgsql': 'django.db.backends.postgresql_psycopg2',
            'mysql': 'django.db.backends.mysql',
            'sqlite3': 'django.db.backends.sqlite3',
            'postgis': 'django.contrib.gis.db.backends.postgis',
        }
        return engines.get(self.get(setting, ""), engines[default])



#!/usr/bin/env python

"""Some web helpers taken from the Pylons camp"""

import os
from django.conf import settings

# SCRIPT_NAME is searched for legacy reasons, typically header will be
# HTTP_SCRIPT_NAME and make its way to environ via the wsgi script
if 'SCRIPT_NAME' in os.environ:
    wsgibasepath = os.environ['SCRIPT_NAME']
elif 'HTTP_SCRIPT_NAME' in os.environ:
    wsgibasepath = os.environ['HTTP_SCRIPT_NAME']
else:
    wsgibasepath=''

def wsgibase():
        return wsgibasepath

def url( relpath ):
        if len(wsgibasepath):
            if relpath and relpath[0]=='/':
                    return "%s%s"%(wsgibasepath,relpath)
            else:
                    return "%s/%s"%(wsgibasepath,relpath)
        return relpath

def deie( html ):
        """try to de - internet explorer - ify the html snippet"""
        if html=="":
                return "&nbsp;"
        return html

def siteurl(request):
    d = request.__dict__
    u = ''
    if 'HTTP_X_FORWARDED_HOST' in d['META']:
        #The request has come from outside, so respect X_FORWARDED_HOST
        u = d['META']['wsgi.url_scheme'] + '://' + d['META']['HTTP_X_FORWARDED_HOST'] + wsgibase() + '/'
    else:
        #Otherwise, its an internal request
        host = d['META'].get('HTTP_HOST')
        if not host:
            host = d['META'].get('SERVER_NAME')
             
        u = d['META']['wsgi.url_scheme'] + '://' + host + wsgibase() + '/' 
    return u
        



from setuptools import setup, find_packages

setup(name='ccg-django-utils',
      version='0.4.2',
      description='Extra CCG code for Django applications.',
      long_description='Extra code used by the Centre for Comparative Genomics in our Django applications.',
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: Web Environment",
          "Framework :: Django",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
          "Topic :: Internet :: WWW/HTTP :: WSGI",
      ],
      keywords='ccg django wsgi utils',
      author='Centre for Comparative Genomics',
      author_email='web@ccg.murdoch.edu.au',
      url='https://github.com/muccg/ccg-django-utils',
      license='GNU General Public License, version 2',
      packages=find_packages(exclude=['examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=['six'],
      )



# -*- coding: utf-8 -*-
### BEGIN COPYRIGHT ###
#
# (C) Copyright 2011, Centre for Comparative Genomics, Murdoch University.
# All rights reserved.
#
# This product includes software developed at the Centre for Comparative Genomics
# (http://ccg.murdoch.edu.au/).
#
# TO THE EXTENT PERMITTED BY APPLICABLE LAWS, YABI IS PROVIDED TO YOU "AS IS,"
# WITHOUT WARRANTY. THERE IS NO WARRANTY FOR YABI, EITHER EXPRESSED OR IMPLIED,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY RIGHTS.
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF YABI IS WITH YOU.  SHOULD
# YABI PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR
# OR CORRECTION.
#
# TO THE EXTENT PERMITTED BY APPLICABLE LAWS, OR AS OTHERWISE AGREED TO IN
# WRITING NO COPYRIGHT HOLDER IN YABI, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
# REDISTRIBUTE YABI AS PERMITTED IN WRITING, BE LIABLE TO YOU FOR DAMAGES, INCLUDING
# ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
# USE OR INABILITY TO USE YABI (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
# DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES
# OR A FAILURE OF YABI TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER
# OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
### END COPYRIGHT ###

from django.http import HttpResponse

class HttpResponseUnauthorized(HttpResponse):
    status_code = 401



import os
import sys
from logging.handlers import TimedRotatingFileHandler


class ParentPathFileHandler(TimedRotatingFileHandler):
    def __init__(self, filename, *args, **kwargs):
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError:
            pass
        TimedRotatingFileHandler.__init__(self, filename, *args, **kwargs)

    def _open(self):
       try:
           return TimedRotatingFileHandler._open(self)
       except IOError as e:
           # If log file can't be opened then don't log to file
           sys.stderr.write("warning: %s\n" % e)
           return open("/dev/null", self.mode)
